package ru.dbotthepony.kstarbound.io

import ru.dbotthepony.kommons.io.readVarInt
import ru.dbotthepony.kommons.io.writeVarInt
import java.io.InputStream
import java.io.OutputStream

class ByteKey(private vararg val bytes: Byte) : Comparable<ByteKey> {
	override fun equals(other: Any?): Boolean {
		return this === other || other is ByteKey && other.bytes.contentEquals(bytes)
	}

	val size: Int
		get() = bytes.size

	operator fun get(index: Int): Byte {
		return bytes[index]
	}

	fun write(stream: OutputStream) {
		stream.writeVarInt(bytes.size)
		stream.write(bytes)
	}

	fun writeRaw(stream: OutputStream) {
		stream.write(bytes)
	}

	override fun hashCode(): Int {
		return bytes.contentHashCode()
	}

	override fun toString(): String {
		return "ByteKey[${bytes.joinToString(", ")}]"
	}

	override fun compareTo(other: ByteKey): Int {
		val cmp = size.compareTo(other.size)
		if (cmp != 0) return cmp

		for (i in bytes.indices) {
			if (bytes[i].toInt() and 0xFF > other.bytes[i].toInt() and 0xFF) {
				return 1
			} else if (bytes[i].toInt() and 0xFF < other.bytes[i].toInt() and 0xFF) {
				return -1
			}
		}

		return 0
	}
}

fun InputStream.readByteKey(): ByteKey {
	return ByteKey(*ByteArray(readVarInt()).also { read(it) })
}

fun OutputStream.writeByteKey(key: ByteKey) {
	key.write(this)
}

fun InputStream.readByteKeyRaw(size: Int): ByteKey {
	return ByteKey(*ByteArray(size).also { read(it) })
}

fun OutputStream.writeRawByteKey(key: ByteKey) {
	key.writeRaw(this)
}
